{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "from datetime import datetime\n",
    "def numerical_derivative(f,x):\n",
    "    \n",
    "    it = np.nditer(x, flags=['multi_index'], op_flags=['readwrite'])\n",
    "    delta = 1e-5\n",
    "    grad = np.zeros_like(x)\n",
    "    \n",
    "    while not it.finished:\n",
    "        idx = it.multi_index\n",
    "        tmp_val = x[idx]\n",
    "        \n",
    "        x[idx] = float(tmp_val)+delta\n",
    "        fx1 = f(x)\n",
    "        \n",
    "        x[idx] = tmp_val-delta\n",
    "        fx2 = f(x)\n",
    "        \n",
    "        grad[idx] = (fx1-fx2)/(2*delta)\n",
    "        \n",
    "        x[idx] = tmp_val\n",
    "        it.iternext()\n",
    "        \n",
    "    return grad\n",
    "    "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [],
   "source": [
    "class Diabetes:\n",
    "    def __init__(self, name, xdata, tdata, i_node, h1_node, o_node, lr, iter_count):\n",
    "        \n",
    "        if xdata.ndim == 1:\n",
    "            raise Exception(\"<__init__> vector is not permissioned\")\n",
    "        \n",
    "        # save parameter\n",
    "        self.name = name\n",
    "        \n",
    "        self.xdata = xdata\n",
    "        self.tdata = tdata\n",
    "        \n",
    "        self.i_node = i_node\n",
    "        self.h1_node = h1_node\n",
    "        self.o_node = o_node\n",
    "        \n",
    "        self.lr = lr\n",
    "        self.iter_count = iter_count\n",
    "        \n",
    "        # assign W,b\n",
    "        self.W2 = np.random.randn(i_node, h1_node)\n",
    "        self.b2 = np.random.randn(h1_node)\n",
    "        \n",
    "        self.W3 = np.random.randn(h1_node, o_node)\n",
    "        self.b3 = np.random.randn(o_node)\n",
    "        \n",
    "        print(\"class is created\")\n",
    "        \n",
    "    def sigmoid(self, x):\n",
    "        return 1/(1+np.exp(-x))\n",
    "    \n",
    "    def feed_forward_solo_data(self, xdata, tdata):\n",
    "        \n",
    "        y = self.feed_forward_cal(xdata)      \n",
    "        return -np.sum(tdata*np.log(y+1e-7)+(1-tdata)*np.log((1-y)+1e-7))\n",
    "    \n",
    "    def feed_forward_multiple_data(self):\n",
    "        y = self.feed_forward_cal(self.xdata)\n",
    "        return -np.sum(self.tdata*np.log(y+1e-7)+(1-self.tdata)*np.log((1-y)+1e-7))\n",
    "    \n",
    "    def feed_forward_cal(self, xdata):\n",
    "        \n",
    "        Z2 = np.dot(xdata, self.W2)+ self.b2\n",
    "        A2 = self.sigmoid(Z2)\n",
    "        \n",
    "        Z3 = np.dot(A2, self.W3) + self.b3\n",
    "        y = A3 = self.sigmoid(Z3)\n",
    "        \n",
    "        return y\n",
    "    \n",
    "    def accuracy(self, test_x_data, test_t_data):\n",
    "        \n",
    "        if test_x_data.ndim != 2:\n",
    "            raise Exception(\"<accuracy> only matrix can be used\")\n",
    "        \n",
    "        matched_list = []\n",
    "        not_matched_list = []\n",
    "        \n",
    "        for i in range(len(test_x_data)):\n",
    "            y,predict_res = self.predict(test_x_data[i])\n",
    "            \n",
    "            if predict_res == test_t_data[i]:\n",
    "                matched_list.append(test_x_data[i])\n",
    "            else:\n",
    "                not_matched_list.append(test_x_data[i])\n",
    "        \n",
    "        accuracy_val = len(matched_list) / len(test_x_data)\n",
    "        \n",
    "        return accuracy_val, matched_list, not_matched_list\n",
    "            \n",
    "    def predict(self, test_x_data):\n",
    "        \n",
    "        if test_x_data.ndim != 1:\n",
    "            raise Exception(\"<predict> only vector can be used\")\n",
    "        \n",
    "        y = self.feed_forward_cal(test_x_data)\n",
    "        \n",
    "        if y >= 0.5 :\n",
    "            return (y,1)\n",
    "        else:\n",
    "            return (y,0)\n",
    "      \n",
    "    def train(self):\n",
    "        start = datetime.now()\n",
    "        \n",
    "#         print(\"start  loss: \",self.feed_forward_multiple_data())\n",
    "        \n",
    "        for i in range(self.iter_count):\n",
    "            \n",
    "            print(\"---------------------------------------------\")\n",
    "            print(\"step \",i,\"loss: \",self.feed_forward_multiple_data())\n",
    "            print(\"---------------------------------------------\")\n",
    "            for j, (xdata, tdata) in enumerate(zip(self.xdata, self.tdata)):\n",
    "                \n",
    "               \n",
    "                f = lambda x : self.feed_forward_solo_data(xdata, tdata)\n",
    "            \n",
    "                self.W2 -= numerical_derivative(f, self.W2) * self.lr\n",
    "                self.b2 -= numerical_derivative(f, self.b2) * self.lr\n",
    "\n",
    "                self.W3 -= numerical_derivative(f, self.W3) * self.lr\n",
    "                self.b3 -= numerical_derivative(f, self.b3) * self.lr\n",
    "                \n",
    "                \n",
    "                if j % 200 == 0 : \n",
    "                    \n",
    "                    print(\"j:\", j,  xdata, tdata,\"loss: \", self.feed_forward_solo_data(xdata, tdata))\n",
    "                    \n",
    "\n",
    "        print(\"final loss: \",self.feed_forward_multiple_data())\n",
    "                \n",
    "        end = datetime.now()\n",
    "    \n",
    "        print(\"elapsed time => \",end-start)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[[-0.294118   0.487437   0.180328  ... -0.53117   -0.0333333  0.       ]\n",
      " [-0.882353  -0.145729   0.0819672 ... -0.766866  -0.666667   1.       ]\n",
      " [-0.0588235  0.839196   0.0491803 ... -0.492741  -0.633333   0.       ]\n",
      " ...\n",
      " [-0.411765   0.21608    0.180328  ... -0.857387  -0.7        1.       ]\n",
      " [-0.882353   0.266332  -0.0163934 ... -0.768574  -0.133333   0.       ]\n",
      " [-0.882353  -0.0653266  0.147541  ... -0.797609  -0.933333   1.       ]]\n"
     ]
    }
   ],
   "source": [
    "data = np.genfromtxt('diabetes.csv',delimiter=',')\n",
    "print(data)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "xdata.shape (559, 8)\n",
      "tdata.shape (559, 1)\n"
     ]
    }
   ],
   "source": [
    "test_cnt = 200\n",
    "xdata = data[:-test_cnt,:-1]\n",
    "tdata = data[:-test_cnt,-1:]\n",
    "\n",
    "print(\"xdata.shape\",xdata.shape)\n",
    "print(\"tdata.shape\",tdata.shape)\n",
    "\n",
    "test_xdata = data[-test_cnt:, :-1]\n",
    "test_tdata = data[-test_cnt:, -1:]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "class is created\n",
      "---------------------------------------------\n",
      "step  0 loss:  405.9039913117267\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  1.725469072342886\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.7279047608101711\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  0.8812159870780633\n",
      "---------------------------------------------\n",
      "step  1 loss:  319.81424872751063\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.986441492169013\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.7031569750634149\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  0.9109045662769434\n",
      "---------------------------------------------\n",
      "step  2 loss:  299.4624053971311\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.8352009938917787\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.6673025610364569\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  0.9549594211495667\n",
      "---------------------------------------------\n",
      "step  3 loss:  290.2224900390093\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.7614109174908517\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.6336357498446651\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  0.9990592406017036\n",
      "---------------------------------------------\n",
      "step  4 loss:  285.3543693838095\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.7216279514542306\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.6037066540374102\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.0384345406114819\n",
      "---------------------------------------------\n",
      "step  5 loss:  282.42426197815576\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6984527106225196\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.5774925364762561\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.0718836262753697\n",
      "---------------------------------------------\n",
      "step  6 loss:  280.4708299088854\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.684045270519962\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.5546697396441085\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.0995508609403686\n",
      "---------------------------------------------\n",
      "step  7 loss:  279.068081111965\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6745205083409864\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.5348689073086054\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1220669395352099\n",
      "---------------------------------------------\n",
      "step  8 loss:  278.00458158038884\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6678196208093669\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.51773171940702\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.140193247164226\n",
      "---------------------------------------------\n",
      "step  9 loss:  277.16436512474047\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6627999829761259\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.5029270301659061\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1546711664486382\n",
      "---------------------------------------------\n",
      "step  10 loss:  276.47843734326693\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6588062415020781\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.49015631700872087\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1661622328865247\n",
      "---------------------------------------------\n",
      "step  11 loss:  275.90308227900573\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6554529587603738\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4791547981710176\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.175230425433348\n",
      "---------------------------------------------\n",
      "step  12 loss:  275.40924714171194\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6525086027670183\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.46969023542022054\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1823438439859035\n",
      "---------------------------------------------\n",
      "step  13 loss:  274.9769099416785\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6498313407805854\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4615604793461521\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1878845936196678\n",
      "---------------------------------------------\n",
      "step  14 loss:  274.59187695604805\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6473325821949648\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.45459043654168996\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.192161318050101\n",
      "---------------------------------------------\n",
      "step  15 loss:  274.24385540694647\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6449558839160953\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.44862889147951945\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1954217016475053\n",
      "---------------------------------------------\n",
      "step  16 loss:  273.92523952009907\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6426645594335445\n"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.44354543855480333\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.197863758711217\n",
      "---------------------------------------------\n",
      "step  17 loss:  273.6303176306541\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6404343011975095\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4392276589580118\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1996455021153989\n",
      "---------------------------------------------\n",
      "step  18 loss:  273.35473898482405\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.638248726899811\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4355785994386069\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2008929667998558\n",
      "---------------------------------------------\n",
      "step  19 loss:  273.09514657605786\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6360966488200148\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4325145639946895\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2017067364499692\n",
      "---------------------------------------------\n",
      "step  20 loss:  272.84891930277627\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6339703686644078\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42996320409396027\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2021671847206892\n",
      "---------------------------------------------\n",
      "step  21 loss:  272.6139878670863\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6318645892369931\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4278618806371807\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2023386496443023\n",
      "---------------------------------------------\n",
      "step  22 loss:  272.38870143538816\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6297757019538338\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42615626621030367\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2022727415982946\n",
      "---------------------------------------------\n",
      "step  23 loss:  272.1717298432708\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6277013070972525\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4247991560192779\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.202010957546325\n",
      "---------------------------------------------\n",
      "step  24 loss:  271.96199105953485\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.625639881149373\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42374945772670675\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2015867447395634\n",
      "---------------------------------------------\n",
      "step  25 loss:  271.7585968310823\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6235905394495367\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42297133364048595\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.2010271301833513\n",
      "---------------------------------------------\n",
      "step  26 loss:  271.5608115671361\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6215528625346552\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4224334718799603\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.200354008601269\n",
      "---------------------------------------------\n",
      "step  27 loss:  271.3680209586428\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6195267663476134\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42210846647004785\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1995851622833715\n",
      "---------------------------------------------\n",
      "step  28 loss:  271.1797078317784\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6175124037997626\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42197228928551894\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1987350704558881\n",
      "---------------------------------------------\n",
      "step  29 loss:  270.9954334163307\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6155100893599532\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42200383940160197\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1978155532252877\n",
      "---------------------------------------------\n",
      "step  30 loss:  270.81482270694266\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6135202410776507\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4221845577715704\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.196836285301803\n",
      "---------------------------------------------\n",
      "step  31 loss:  270.6375529384575\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.611543336075746\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4224980969777021\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1958052069742298\n",
      "---------------------------------------------\n",
      "step  32 loss:  270.46334445012246\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6095798766137354\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.422930037591005\n"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1947288538224448\n",
      "---------------------------------------------\n",
      "step  33 loss:  270.2919533936704\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6076303645343966\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42346764399620984\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1936126218445409\n",
      "---------------------------------------------\n",
      "step  34 loss:  270.12316587721443\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6056952823898778\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42409965367705793\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1924609812798825\n",
      "---------------------------------------------\n",
      "step  35 loss:  269.9567932297262\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6037750798922542\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42481609499754863\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1912776493491057\n",
      "---------------------------------------------\n",
      "step  36 loss:  269.7926681514404\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.6018701646063442\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.42560812924093805\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1900657301113469\n",
      "---------------------------------------------\n",
      "step  37 loss:  269.63064156363566\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5999808960051304\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4264679134125819\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1888278279256868\n",
      "---------------------------------------------\n",
      "step  38 loss:  269.47058001598793\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5981075821374436\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4273884807950046\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1875661395194832\n",
      "---------------------------------------------\n",
      "step  39 loss:  269.31236354191685\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.596250478351648\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4283636367845446\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1862825288840295\n",
      "---------------------------------------------\n",
      "step  40 loss:  269.1558838731767\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5944097875660509\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4293878678509509\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1849785881626884\n",
      "---------------------------------------------\n",
      "step  41 loss:  269.0010429459311\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.592585661710388\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4304562618697991\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.183655687183645\n",
      "---------------------------------------------\n",
      "step  42 loss:  268.8477516432532\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.590778203992624\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.43156443825150015\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1823150137810348\n",
      "---------------------------------------------\n",
      "step  43 loss:  268.69592873213367\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5889874717735984\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4327084865807505\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1809576065469924\n",
      "---------------------------------------------\n",
      "step  44 loss:  268.5454999580768\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5872134797956465\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.43388491270353546\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1795843814411373\n",
      "---------------------------------------------\n",
      "step  45 loss:  268.3963972712738\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5854562036503175\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4350905912040945\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1781961533973178\n",
      "---------------------------------------------\n",
      "step  46 loss:  268.24855816085244\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5837155833169599\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.436322723569871\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1767936537874226\n",
      "---------------------------------------------\n",
      "step  47 loss:  268.10192507962034\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5819915267124757\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.43757880126432624\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1753775445697998\n",
      "---------------------------------------------\n",
      "step  48 loss:  267.9564449430114\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5802839131411769\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.4388565731322903\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.173948429656312\n"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "---------------------------------------------\n",
      "step  49 loss:  267.8120686918767\n",
      "---------------------------------------------\n",
      "j: 0 [-0.294118    0.487437    0.180328   -0.292929    0.          0.00149028\n",
      " -0.53117    -0.0333333 ] [0.] loss:  0.5785925966183921\n",
      "j: 200 [ 0.         0.0854271  0.114754  -0.59596    0.        -0.186289\n",
      " -0.394535  -0.633333 ] [1.] loss:  0.44015401660146547\n",
      "j: 400 [-0.529412  0.155779  0.180328  0.        0.       -0.138599 -0.745517\n",
      " -0.166667] [0.] loss:  1.1725068640502074\n",
      "final loss:  267.66875090829484\n",
      "elapsed time =>  0:04:26.857999\n"
     ]
    }
   ],
   "source": [
    "i_node = 8\n",
    "h1_node = 30\n",
    "o_node = 1\n",
    "lr = 0.01\n",
    "iter_count = 50\n",
    "\n",
    "obj = Diabetes(\"Diabetes\", xdata,tdata, i_node, h1_node, o_node, lr, iter_count)\n",
    "obj.train()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "accuracy, matched_list, not_matched_list = obj.accuracy(test_xdata,test_tdata)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "0.705\n"
     ]
    }
   ],
   "source": [
    "print(accuracy)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.7.3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
